package com.mockitotutorial.happyhotel.booking;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

class Test03ReturnCustomValues {

	private BookingService bookingService;
	private PaymentService paymentServiceMock;
	private RoomService roomServiceMock;
	private BookingDAO bookingDAOMock;
	private MailSender mailSenderMock;

	@BeforeEach
	void setup(){
		paymentServiceMock = mock(PaymentService.class);
		roomServiceMock = mock(RoomService.class);
		bookingDAOMock = mock(BookingDAO.class);
		mailSenderMock = mock(MailSender.class);
		bookingService = new BookingService(paymentServiceMock, roomServiceMock, bookingDAOMock, mailSenderMock);
	}

	@Test
	void should_CountAvailablePlaces_When_OneRoomAvailable() {
		//given
		when(roomServiceMock.getAvailableRooms()).thenReturn(Collections.singletonList(new Room("Room2", 2)));
		int expected = 2;
		//when
		int actual = bookingService.getAvailablePlaceCount();
		//then
		assertEquals(expected, actual);
	}
	@Test
	void should_CountAvailablePlaces_When_MultipleRoomsAvailable() {
		//given
		List<Room> rooms = Arrays.asList(new Room("Room 1", 2), new Room("Room 2", 5));
		when(roomServiceMock.getAvailableRooms()).thenReturn(rooms);
		int expected = 7;
		//when
		int actual = bookingService.getAvailablePlaceCount();
		//then
		assertEquals(expected, actual);
	}

}
